#include <stdio.h>
#include <stdlib.h>

int main(void)
{
 FILE * 	binfile;
 unsigned char  buffer[256];
 unsigned char 	test1[8] =			/* jal instruction test	*/
		{ 0x00, 0x00, 0x1f, 0x38,	/* jal			*/
		  0x02, 0x00, 0x20, 0x00};	/* 0x00800008		*/
 unsigned char	test2[4] =			/* bl instruction test	*/
		{ 0x00, 0x00, 0x00, 0x28};	/* bl 0x008000C 	*/
 unsigned char  test3[8] =			/* relocation test	*/
		{ 0x00, 0x7c, 0x1f, 0x60,	/* mov r0,		*/
		  0xb4, 0x00, 0x80, 0x00};	/* 0x008000b4		*/

 binfile = fopen("target.bin", "r");

 if (binfile == NULL)
    {
     fprintf(stderr, "target.bin does not exist, run \"make target!\"\n");
     goto exit;
    }

 fread(buffer, 1, 256, binfile);

 /*
  * Test 1, JAL instruction
  */
 printf("Test JAL instruction:");
 if (memcmp(buffer, test1, 8))
    {
     printf("Failed\n");
    }
 else
    {
     printf("OK\n");
    }
 
 /*
  * Test 2, BL instruction
  */
 printf("Test BL instruction:");
 if (memcmp(buffer + 8, test2, 4))
    {
     printf("Failed\n");
    }
 else
    {
     printf("OK\n");
    }
 
 /*
  * Test 3, relocation problem
  * This happens when we use "arc-elf-ld -r" to create a relocatable
  * elf object file. The relocation entries in the newly created file
  * are wrong.
  *
  * Test case : We have a core.r made by linking crt0.o main.o test1.o
  *		and printf.o together.
  *		arc-elf-ld -r crt0.o main.o test1.o printf.o -o core.r
  * In test1.c, we have a statement as followed:
  * 	printf("test1\n");
  * This C statement is translated into machine codes as followed:
  *	mov	r0, 0x80000a4		# r0 = offset of "test1\n" string
  *					# in ".rodata" section
  *	bl	0x800094		# 0x80094 is printf subroutine
  *
  * Note that the offset of "test1\n" string should be
  *	start address of ".rodata" section of core.r +
  *     Length of ".rodata" section in main.o +
  *	offset  of "test1\n" string in ".rodata" section of test1.o
  *
  * In this case, the base address of ".rodata" of core.r is 0x008000a4.
  * Length of ".rodata" section of main.o is 0x00000010.
  * offset of "test1\n" string in ".rodata" section of test1.o is 0x00000000.
  * The real offset of "test1\n" string after final linking should be
  * 0x008000a4 + 0x00000010 + 0x000000 = 0x008000b4
  */
 printf("Test relocation problem:");
 if (memcmp(buffer + 48, test3, 8))
    {
     printf("Failed\n");
    }
 else
    {
     printf("OK\n");
    }

 exit:
 fclose(binfile);
 return(0);

}
